/*******************************************************************************
*
*      "cs46xx_wrapper.h" --  Cirrus Logic-Crystal CS46XX linux audio driver.
*
*      Copyright (C) 2000,2001  Cirrus Logic Corp.  
*            -- tom woller (twoller@crystal.cirrus.com) or
*               (pcaudio@crystal.cirrus.com).
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 2 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* 01/11/2001 trw - new file from cs4281 wrapper code. 
*
*******************************************************************************/
#ifndef __CS46XX_WRAPPER22_H
#define __CS46XX_WRAPPER22_H

/* wrapper for 2.2 kernel */

#include <asm/page.h>
#include <linux/mm.h>
#include <asm/io.h>
#include <linux/malloc.h>
#include <linux/tqueue.h>

#define list_for_each(pos, head) \
        for (pos = (head)->next; pos != (head); pos = pos->next)

#if LINUX_VERSION_CODE < 0x020213 /* 2.2.19 is more like 2.4.0 */
#define DECLARE_WAITQUEUE(a, b)	struct wait_queue a = {b, NULL}
#define init_waitqueue_head(a)	init_waitqueue(a)
#define init_MUTEX(a)		*(a) = MUTEX

/* Not sure what version aliases were introduced in, but certainly in 2.91.66.  */
#ifdef MODULE
  #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)
    #define module_init(x)      int init_module(void) __attribute__((alias(#x)));
    #define module_exit(x)      void cleanup_module(void) __attribute__((alias(#x)));
  #else
    #define module_init(x)      int init_module(void) { return x(); }
    #define module_exit(x)      void cleanup_module(void) { x(); }
  #endif
#else
  #define module_init(x)
  #define module_exit(x)
#endif

static __inline__ void list_add_tail(struct list_head *new, struct list_head *head)
{
        __list_add(new, head->prev, head);
}
typedef u32 dma_addr_t;

#endif /* end < 0x020213 */

#define wait_queue_head_t	struct wait_queue *

#define __exit
#define __exitdata
#define __devinit
#define __devinitdata
#define __devexit
#define __devexitdata

#define MODULE_DEVICE_TABLE(foo,bar)

#define pci_dma_supported(dev, mask) 1

#define PCI_ANY_ID (~0)

#define RSRCISIOREGION(dev,num) ((dev)->base_address[(num)] != 0 && \
	 ((dev)->base_address[(num)] & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_IO)
#define RSRCISMEMORYREGION(dev,num) ((dev)->base_address[(num)] != 0 && \
	 ((dev)->base_address[(num)] & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_MEMORY)
#define RSRCADDRESS(dev,num) (((dev)->base_address[(num)]) & PCI_BASE_ADDRESS_MEM_MASK)

#define PCI_GET_DRIVER_DATA cs_pci_get_driver_data
#define PCI_SET_DRIVER_DATA cs_pci_set_driver_data

#define PCI_SET_DMA_MASK(dev,data)

#define pci_enable_device cs_pci_enable_device
#define pci_register_driver cs_pci_register_driver
#define pci_unregister_driver cs_pci_unregister_driver

#define pci_for_each_dev(dev) \
        for(dev = pci_devices; dev; dev = dev->next)

struct pci_device_id {
        unsigned int vendor, device;
        unsigned int subvendor, subdevice;
        unsigned int class, class_mask;
        unsigned long driver_data;
};

struct pci_driver {
        struct list_head node;
        struct pci_dev *dev;
        char *name;
        const struct pci_device_id *id_table;   /* NULL if wants all devices */
        int (*probe)(struct pci_dev *dev, const struct pci_device_id *id);      /* New device inserted */
        void (*remove)(struct pci_dev *dev);    /* Device removed (NULL if not a hot-plug capable driver) */
        void (*suspend)(struct pci_dev *dev);   /* Device suspended */
        void (*resume)(struct pci_dev *dev);    /* Device woken up */
};

const struct pci_device_id * cs_pci_match_device(const struct pci_device_id *ids, struct pci_dev *dev);
int cs_pci_register_driver(struct pci_driver *drv);
void cs_pci_unregister_driver(struct pci_driver *drv);
int cs_pci_enable_device(struct pci_dev *dev);
void * cs_pci_get_driver_data (struct pci_dev *dev);
void cs_pci_set_driver_data (struct pci_dev *dev, void *driver_data);

extern __inline__ int __compat_get_order(unsigned long size)
{
        int order;

        size = (size-1) >> (PAGE_SHIFT-1);
        order = -1;
        do {
                size >>= 1;
                order++;
        } while (size);
        return order;
}

extern __inline__ void *
pci_alloc_consistent(struct pci_dev *hwdev,
                     size_t size, dma_addr_t *dma_handle) {
        void *ret;
        int gfp = GFP_ATOMIC;

        if (hwdev == NULL)
                gfp |= GFP_DMA;
        ret = (void *)__get_free_pages(gfp, __compat_get_order(size));

        if (ret != NULL) {
                memset(ret, 0, size);
                *dma_handle = virt_to_bus(ret);
        }
        return ret;
}

static inline int pci_module_init(struct pci_driver *drv)
{
        int rc = pci_register_driver (drv);

        if (rc > 0)
                return 0;

        /* if we get here, we need to clean up pci driver instance
         * and return some sort of error */
        pci_unregister_driver (drv);
        
        return -ENODEV;
}

#include <asm/page.h>
#include <asm/io.h>
#include <asm/spinlock.h>

#define CS_OWNER 
#define CS_THIS_MODULE 
#if LINUX_VERSION_CODE < 0x020213 /* 2.2.19 kernel is more like 2.4.0 */
#define set_current_state(x) current->state=x
#define __set_current_state(x) set_current_state(x)
#endif
#define cs4x_pgoff(vma) ((vma)->vm_offset >> PAGE_SHIFT)
#define virt_to_page(addr) (mem_map + MAP_NR(addr))
#define cs4x_mem_map_reserve(page) mem_map_reserve((page) - mem_map)
#define cs4x_mem_map_unreserve(page) mem_map_unreserve((page) - mem_map)
#define free_dmabuf(card, dmabuf) \
	free_pages((unsigned long)(dmabuf)->rawbuf, (dmabuf)->buforder);
#define free_dmabuf2(card, dmabuf) \
	free_pages((unsigned long)(dmabuf)->tmpbuff, (dmabuf)->buforder_tmpbuff);

LIST_HEAD(cs_pci_drivers);

struct pci_driver_mapping {
        struct pci_dev *dev;
        struct pci_driver *drv;
        void *driver_data;
};

#define PCI_MAX_MAPPINGS 16
#endif
